@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -593,9 +593,7 @@ void ExternalToolRunner::run()
     if (m_tool->modifiesCurrentDocument()) {
         if (IDocument *document = EditorManager::currentDocument()) {
             m_expectedFileName = document->filePath();
-            bool cancelled = false;
-            DocumentManager::saveModifiedDocuments(QList<IDocument *>() << document, &cancelled);
-            if (cancelled) {
+            if (!DocumentManager::saveModifiedDocument(document)) {
                 deleteLater();
                 return;
             }
@@ -702,7 +700,7 @@ ExternalToolManager::ExternalToolManager()
 
     d->m_configureSeparator = new QAction(this);
     d->m_configureSeparator->setSeparator(true);
-    d->m_configureAction = new QAction(tr("Configure..."), this);
+    d->m_configureAction = new QAction(Core::ICore::msgShowOptionsDialog(), this);
     connect(d->m_configureAction, SIGNAL(triggered()), this, SLOT(openPreferences()));
 
     // add the external tools menu
