@@ -1,6 +1,6 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of Qt Creator.
@@ -55,16 +55,10 @@ class QbsFileNode : public ProjectExplorer::FileNode
 public:
     QbsFileNode(const QString &filePath, const ProjectExplorer::FileType fileType, bool generated,
                 int line);
-    int line() const { return m_line; }
-
-    void setLine(int l);
 
     QString displayName() const;
 
     bool update(const qbs::CodeLocation &loc);
-
-private:
-    int m_line;
 };
 
 // ---------------------------------------------------------------------------
@@ -80,9 +74,9 @@ class QbsBaseProjectNode : public ProjectExplorer::ProjectNode
 public:
     explicit QbsBaseProjectNode(const QString &path);
 
-    bool hasBuildTargets() const;
+    bool showInSimpleTree() const;
 
-    QList<ProjectAction> supportedActions(Node *node) const;
+    QList<ProjectExplorer::ProjectAction> supportedActions(Node *node) const;
 
     bool canAddSubProject(const QString &proFilePath) const;
 
@@ -144,7 +138,7 @@ public:
     explicit QbsProductNode(const qbs::ProductData &prd);
 
     bool isEnabled() const;
-    bool hasBuildTargets() const;
+    bool showInSimpleTree() const;
 
     void setQbsProductData(const qbs::ProductData prd);
     const qbs::ProductData qbsProductData() const { return m_qbsProductData; }
@@ -178,6 +172,7 @@ public:
     const qbs::Project qbsProject() const;
     const qbs::ProjectData qbsProjectData() const;
 
+    bool showInSimpleTree() const;
 private:
     void ctor();
 
